//+------------------------------------------------------------------+
//|                                              stochastic momentum |
//|                                                           mladen |
//|                                                                  |
//| Stochastic momentum developed by William Blau                    |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers    4
#property indicator_color1     DodgerBlue
#property indicator_color2     Red
#property indicator_color3     White
#property indicator_color4     Red
#property indicator_width1     2
#property indicator_width2     2
#property indicator_width3     1
#property indicator_width4     1
#property indicator_level1     0
#property indicator_level2     50
#property indicator_level3     -50
#property indicator_levelcolor DarkSlateGray

//
//
//
//
//

extern int    Length           = 14;
extern int    Smooth1          = 10;
extern int    Smooth2          = 14;
extern int    Signal           = 5;
extern int    Price            = PRICE_CLOSE;
extern double osmaMultiplier   = 1;
extern bool   alertsOn         = true;
extern bool   alertsOnCurrent  = true;
extern bool   alertsMessage    = true;
extern bool   alertsSound      = false;
extern bool   alertsNotify     = false;
extern bool   alertsEmail      = false;
extern string soundfile        = "alert2.wav"; 
extern bool   ShowArrows       = false;
extern string arrowsIdentifier = "smi Arrows";
extern color  arrowsUpColor    = LimeGreen;
extern color  arrowsDnColor    = Red;

//
//
//
//
//

double smo[];
double sms[];
double upH[];
double dnH[];
double osmo[];
double trend[];
double tBuffer[][6];

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(6);
   SetIndexBuffer(0,smo); SetIndexLabel(0,"Stochastic momentum");
   SetIndexBuffer(1,sms); SetIndexLabel(1,"Stochastic momentum signal");
   SetIndexBuffer(2,upH); SetIndexStyle(2,DRAW_HISTOGRAM);
   SetIndexBuffer(3,dnH); SetIndexStyle(3,DRAW_HISTOGRAM);
   SetIndexBuffer(4,osmo);
   SetIndexBuffer(5,trend);

   //
   //
   //
   //
   //

   Length  = MathMax(Length ,1);
   Smooth1 = MathMax(Smooth1,1);
   Smooth2 = MathMax(Smooth2,1);
   
   //
   //
   //
   //
   //
   
   IndicatorShortName (" Stochastic momentum Osma ("+Length+","+Smooth1+","+Smooth2+","+Price+")");
   return(0);
}
int deinit()
{
   deleteArrows();
   
return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

#define ema10 0
#define ema11 1
#define ema12 2
#define ema20 3
#define ema21 4
#define ema22 5

int start()
{
   int counted_bars=IndicatorCounted();
   int i,r,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         if (ArrayRange(tBuffer,0) != Bars) ArrayResize(tBuffer,Bars);

         double alpha1 = 2.0 /(1.0+Smooth1);
         double alpha2 = 2.0 /(1.0+Smooth2);
         double alphas = 2.0 /(1.0+Signal);
         
   //
   //
   //
   //
   //
   //

   for(i=limit,r=Bars-i-1; i >= 0; i--,r++)
   {
      double hh = High[iHighest(NULL,0,MODE_HIGH,Length,i)];
      double ll =  Low[ iLowest(NULL,0,MODE_LOW ,Length,i)];
      double pr = iMA(NULL,0,1,0,MODE_SMA,Price,i);

         tBuffer[r][ema10] = pr - 0.5*(hh+ll);
         tBuffer[r][ema20] =           hh-ll;
         if (i>=Bars-3)
         {
            tBuffer[r][ema11] = tBuffer[r][ema10];
            tBuffer[r][ema12] = tBuffer[r][ema10];
            tBuffer[r][ema21] = tBuffer[r][ema20];
            tBuffer[r][ema22] = tBuffer[r][ema20];
            continue;
         }
      
      //
      //
      //
      //
      //
      
      tBuffer[r][ema11] = tBuffer[r-1][ema11] + alpha1*(tBuffer[r][ema10]-tBuffer[r-1][ema11]);
      tBuffer[r][ema12] = tBuffer[r-1][ema12] + alpha2*(tBuffer[r][ema11]-tBuffer[r-1][ema12]);
      tBuffer[r][ema21] = tBuffer[r-1][ema21] + alpha1*(tBuffer[r][ema20]-tBuffer[r-1][ema21]);
      tBuffer[r][ema22] = tBuffer[r-1][ema22] + alpha2*(tBuffer[r][ema21]-tBuffer[r-1][ema22]);

         smo[i] = 100.00 * tBuffer[r][ema12] / (0.5 * tBuffer[r][ema22]);
         if (Signal>1)
            sms[i]  = sms[i+1]+alphas*(smo[i]-sms[i+1]);
            osmo[i] = (smo[i]-sms[i])*osmaMultiplier;
            
            //
            //
            //
            //
            //
            
            upH[i]   = EMPTY_VALUE;
            dnH[i]   = EMPTY_VALUE;
            trend[i] = trend[i+1];
              if (osmo[i]>0)     trend[i] = 1;
              if (osmo[i]<0)     trend[i] =-1;
              if (trend[i] == 1) upH[i] = osmo[i]; 
              if (trend[i] ==-1) dnH[i] = osmo[i]; 
              manageArrow(i);
              
   }
   
   //
   //
   //
   //
   //
      
   if (alertsOn)
   {
     if (alertsOnCurrent)
          int whichBar = 0;
     else     whichBar = 1;
     if (trend[whichBar] != trend[whichBar+1])
     if (trend[whichBar] == 1)
           doAlert("Buy");
     else  doAlert("Sell");       
   }
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," SMI Osma ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(StringConcatenate(Symbol(), Period() ," SMI Osma " +" "+message));
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," SMI Osma "),message);
             if (alertsSound)   PlaySound(soundfile);
      }
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void manageArrow(int i)
{
   if (ShowArrows)
   {
      deleteArrow(Time[i]);
      if (trend[i]!=trend[i+1])
      {
         if (trend[i] == 1) drawArrow(i,arrowsUpColor,225,false);
         if (trend[i] ==-1) drawArrow(i,arrowsDnColor,226,true);
      }
   }
}               

//
//
//
//
//

void drawArrow(int i,color theColor,int theCode,bool up)
{
   string name = arrowsIdentifier+":"+Time[i];
   double gap  = 3.0*iATR(NULL,0,20,i)/4.0;   
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i]+ gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i] - gap);
}

//
//
//
//
//

void deleteArrows()
{
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
}

//
//
//
//
//

void deleteArrow(datetime time)
{
   string lookFor = arrowsIdentifier+":"+time; ObjectDelete(lookFor);
}